package com.fr.plugin.wechat.sync;

import com.fr.base.PropertiesUtils;
import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.intelli.record.Focus;
import com.fr.intelli.record.Original;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.context.PluginContexts;
import com.fr.plugin.transform.FunctionRecorder;
import com.fr.plugin.wechat.sync.schedule.SyncThread;
import com.fr.scheduler.QuartzContext;
import com.fr.scheduler.ScheduleJobManager;
import com.fr.stable.StringUtils;
import com.fr.stable.fun.Authorize;
import com.fr.third.v2.org.quartz.CronScheduleBuilder;
import com.fr.third.v2.org.quartz.JobKey;
import com.fr.third.v2.org.quartz.SchedulerException;
import com.fr.third.v2.org.quartz.TriggerBuilder;

import java.util.HashMap;
import java.util.Map;

/**
 * @Author fr.open
 * @Date 2021/7/29
 * @Description
 **/
@FunctionRecorder
@Authorize(callSignKey = PluginConstants.PLUGIN_ID)
public class SyncFilter  extends AbstractGlobalRequestFilterProvider {
    @Override
    public String filterName() {
        return "sync";
    }

    @Override
    @Focus(id = PluginConstants.PLUGIN_ID, text = "微信同步", source = Original.PLUGIN)
    public String[] urlPatterns() {
        if (PluginContexts.currentContext().isAvailable()) {
            initSchedule();
        }
        return new String[]{"/syncWechat"};
    }


    public static void initSchedule() {
        JobKey jobKey = new JobKey("syncWechat", "syncGroup");
        try {
            if (QuartzContext.getInstance().getScheduler().checkExists(jobKey)) {
                ScheduleJobManager.getInstance().removeJob(jobKey.getName(), jobKey.getGroup());
            }
        } catch (SchedulerException e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }

        addSchedule(jobKey);
    }

    private static void addSchedule(JobKey jobKey) {
        Map<String, Object> param = new HashMap();
        TriggerBuilder triggerBuilder = TriggerBuilder.newTrigger();
        triggerBuilder.forJob(jobKey.getName(), jobKey.getGroup()).withIdentity(jobKey.getName(), jobKey.getGroup()).startNow();
        String cron = PropertiesUtils.getProperties("wechat").getProperty("cron");
        if (StringUtils.isBlank(cron)) {
            FineLoggerFactory.getLogger().error("cron is null schedule start failed");
            return;
        }
        CronScheduleBuilder schedule = CronScheduleBuilder.cronSchedule(cron);
        triggerBuilder.withSchedule(schedule);
        try {
            ScheduleJobManager.getInstance().addJob(jobKey.getName(), jobKey.getGroup(), "sync job", SyncThread.class, triggerBuilder.build(), param);
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
    }
}
