package com.fr.plugin.wechat.sync.dao;

import com.fr.log.FineLoggerFactory;
import com.fr.plugin.wechat.sync.SyncDBAccessProvider;
import com.fr.plugin.wechat.sync.entity.TagRoleEntity;
import com.fr.stable.db.action.DBAction;
import com.fr.stable.db.dao.DAOContext;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.restriction.RestrictionFactory;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * @Author fr.open
 * @Date 2021/7/29
 * @Description
 **/
public class TagRoleService {

    public static TagRoleEntity create(TagRoleEntity entity) {
        try {
            return SyncDBAccessProvider.getDbAccessor().runDMLAction(new DBAction<TagRoleEntity>() {

                @Override
                public TagRoleEntity run(DAOContext daoContext) throws Exception {
                    entity.setId(UUID.randomUUID().toString());
                    daoContext.getDAO(TagRoleDao.class).add(entity);
                    return entity;
                }
            });
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
        return entity;
    }

    public static TagRoleEntity delete(String tagId) {
        try {
            return SyncDBAccessProvider.getDbAccessor().runDMLAction(new DBAction<TagRoleEntity>() {

                @Override
                public TagRoleEntity run(DAOContext daoContext) throws Exception {
                    TagRoleDao dao = daoContext.getDAO(TagRoleDao.class);
                    TagRoleEntity entity = dao.findOne(QueryFactory.create().addRestriction(RestrictionFactory.eq("tagId", tagId)));
                    dao.remove(entity.getId());
                    return entity;
                }
            });
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
        return null;
    }

    public static Map<String ,String> getLinkMap() {
        Map<String ,String> map = new HashMap<>();
        try {
            List<TagRoleEntity> tagRoleEntities = SyncDBAccessProvider.getDbAccessor().runDMLAction(new DBAction<List<TagRoleEntity>>() {
                @Override
                public List<TagRoleEntity> run(DAOContext daoContext) throws Exception {
                    TagRoleDao dao = daoContext.getDAO(TagRoleDao.class);
                    return dao.find(QueryFactory.create());
                }
            });
           return tagRoleEntities.stream().collect(Collectors.toMap(e->e.getTagId(),e->e.getRoleId(),(k1,k2)->k2));
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
        return map;
    }
}
