package com.fr.plugin.patrol.sync.schedule;

import com.fr.base.TableData;
import com.fr.cluster.core.ClusterNode;
import com.fr.decision.authority.AuthorityContext;
import com.fr.decision.authority.data.CustomRole;
import com.fr.decision.authority.data.User;
import com.fr.decision.webservice.service.user.UserMiddleRoleService;
import com.fr.file.TableDataConfig;
import com.fr.general.PropertiesUtils;
import com.fr.general.data.DataModel;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.patrol.sync.HttpUtil;
import com.fr.scheduler.job.FineScheduleJob;
import com.fr.script.Calculator;
import com.fr.stable.query.QueryFactory;
import com.fr.third.v2.org.quartz.JobExecutionContext;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author fr.open
 * @Date 2021/7/29
 * @Description
 **/
public class SyncThread extends FineScheduleJob {

    private DataModel dataModel;

    private String addRole;

    private String getUser;

    private String addUser;

    private String deleteUser;

    private String createId;

    private String findRole;

    private String editRole;


    public SyncThread() {
        String name = PropertiesUtils.getProperties("patrol").getProperty("name");
        FineLoggerFactory.getLogger().info("get table data config is {}", name);

        addRole = PropertiesUtils.getProperties("patrol").getProperty("addRole");
        FineLoggerFactory.getLogger().info("get addRole config is {}", addRole);

        getUser = PropertiesUtils.getProperties("patrol").getProperty("getUser");
        FineLoggerFactory.getLogger().info("get getUser config is {}", getUser);

        addUser = PropertiesUtils.getProperties("patrol").getProperty("addUser");
        FineLoggerFactory.getLogger().info("get addUser config is {}", addUser);

        deleteUser = PropertiesUtils.getProperties("patrol").getProperty("deleteUser");
        FineLoggerFactory.getLogger().info("get deleteUser config is {}", deleteUser);

        createId = PropertiesUtils.getProperties("patrol").getProperty("createId");
        FineLoggerFactory.getLogger().info("get createId config is {}", createId);

        findRole = PropertiesUtils.getProperties("patrol").getProperty("findRole");
        FineLoggerFactory.getLogger().info("get findRole config is {}", findRole);

        editRole = PropertiesUtils.getProperties("patrol").getProperty("editRole");
        FineLoggerFactory.getLogger().info("get editRole config is {}", editRole);
        TableData tableData = TableDataConfig.getInstance().getTableData(name);
        dataModel = tableData.createDataModel(Calculator.createCalculator());
    }

    @Override
    public void run(JobExecutionContext jobExecutionContext, ClusterNode clusterNode) throws Exception {
        List<CustomRole> customRoles = AuthorityContext.getInstance().getCustomRoleController().find(QueryFactory.create());
        customRoles = customRoles.stream().filter(e -> !"superusers".equals(e.getName())).collect(Collectors.toList());
        FineLoggerFactory.getLogger().info("get role size is {}", customRoles);
        Map<String, String> roleMap = getRoleMap();
        for (CustomRole role : customRoles) {
            String roleCode = role.getId().replace("-", "");
            if (roleMap.containsKey(roleCode)) {
               //角色名称不同则更新角色名称
                if (!role.getName().equals(roleMap.get(roleCode))) {
                    editRole(roleCode, role.getName());
                }
            } else {
                //首先新建角色
                roleCode = addRole(role);
            }
            //系统下的角色用户
            Map<String, String> users = getUsers(role.getId());
            //门户角色下用户
            Set<String> roleUsers = getRoleUser(roleCode);

            Set<String> delete = roleUsers.stream().filter(e -> !users.keySet().contains(e)).collect(Collectors.toSet());
            deleteRoleUser(roleCode, delete);

            Set<String> add = users.keySet().stream().filter(e -> !delete.contains(e)).collect(Collectors.toSet());
            addRoleUser(roleCode, add);

        }
    }

    private void editRole(String code, String name) {
        String res = HttpUtil.sendPost(editRole, null, new JSONObject().put("roleId", code).put("roleName", name).put("createId", createId));
        FineLoggerFactory.getLogger().info("edit role res is {}", res);
    }

    private Map<String, String> getRoleMap() {
        String res = HttpUtil.sendPost(findRole, null, new JSONObject().put("createId", createId));
        FineLoggerFactory.getLogger().info("get role res is {}", res);
        JSONObject object = new JSONObject(res);
        Map<String, String> map = new HashMap<>();
        JSONArray date = object.getJSONArray("data");
        for (int i = 0; i < date.size(); i++) {
            JSONObject entries = date.getJSONObject(i);
            map.put(entries.getString("roleId"), entries.getString("roleName"));
        }
        return map;
    }

    private void addRoleUser(String roleCode, Set<String> add) {
        if (add == null || add.isEmpty()) {
            return;
        }
        Iterator<String> it = add.iterator();
        while (it.hasNext()) {
            String next = it.next();
            String res = HttpUtil.sendPost(addUser, null, new JSONObject().put("roleId", roleCode).put("authorizationUser", next).put("createId", createId));
            FineLoggerFactory.getLogger().info("delete role users res is {}", res);
        }
    }

    private void deleteRoleUser(String roleCode, Set<String> delete) {
        if (delete == null || delete.isEmpty()) {
            return;
        }
        Iterator<String> it = delete.iterator();
        while (it.hasNext()) {
            String next = it.next();
            String res = HttpUtil.sendPost(deleteUser, null, new JSONObject().put("roleId", roleCode).put("authorizationUser", next).put("createId", createId));
            FineLoggerFactory.getLogger().info("delete role users res is {}", res);
        }
    }

    private Set<String> getRoleUser(String roleCode) {
        String res = HttpUtil.sendPost(getUser, null, new JSONObject().put("roleId", roleCode).put("createId", createId));
        FineLoggerFactory.getLogger().info("get role users res is {}", res);
        JSONObject object = new JSONObject(res);
        JSONArray date = object.getJSONObject("data").getJSONArray("userAuth");
        Set<String> users = new HashSet<>();
        for (int i = 0; i < date.size(); i++) {
            JSONObject entries = date.getJSONObject(i);
            users.add(entries.getString("userCode"));
        }
        return users;
    }

    private String addRole(CustomRole role) {
        String code = role.getId().replaceAll("-", "");
        JSONObject object = new JSONObject().put("roleId", code).put("roleName", role.getName()).put("createId", createId);
        String res = HttpUtil.sendPost(addRole, null, object);
        FineLoggerFactory.getLogger().info("add role res is {}", res);
        return code;
    }


    private Map<String, String> getUsers(String roleId) throws Exception {
        Set<String> ids = UserMiddleRoleService.getInstance().getAllUserIdsByCustomRole(roleId);
        List<User> all = AuthorityContext.getInstance().getUserController().find(QueryFactory.create());
        Set<String> usersMap = all.stream().filter(e -> ids.contains(e.getId())).map(e -> e.getUserName()).collect(Collectors.toSet());
        Map<String, String> result = new HashMap<>();
        try {
            for (int i = 0; i < dataModel.getRowCount(); i++) {
                Object valueAt = dataModel.getValueAt(i, 4);
                if (valueAt != null && usersMap.contains(valueAt.toString())) {
                    result.put(String.valueOf(dataModel.getValueAt(i, 7)), valueAt.toString());
                }
            }
        } catch (Exception e) {

        }
        return result;
    }
}
